#include <stdlib.h>
#include <string.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
#define YYPREFIX "yy"
#line 22 "parse.y"
#include <sys/types.h>

#include <grp.h>
#include <libgen.h>
#include <limits.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

#include "logfmon.h"

struct macros	 parse_macros;

struct cfgfiles  parse_filestack;
struct cfgfile	*parse_file;

int		 yyparse(void);
void 		 yyerror(const char *, ...);

int
parse_conf(const char *path)
{
	struct macro	*macro;
	FILE		*f;

        if ((f = fopen(path, "r")) == NULL)
                return (-1);
	
	ARRAY_INIT(&parse_filestack);
	parse_file = xmalloc(sizeof *parse_file);

	parse_file->f = f;
	parse_file->line = 1;
	parse_file->path = path;

	TAILQ_INIT(&parse_macros);

	macro = xmalloc(sizeof *macro);
	strlcpy(macro->name, "%pid", sizeof macro->name);
	macro->type = MACRO_NUMBER;
	macro->value.num = getpid();
	TAILQ_INSERT_HEAD(&parse_macros, macro, entry);

        yyparse();

	ARRAY_FREE(&parse_filestack);
	xfree(parse_file);

	while (!TAILQ_EMPTY(&parse_macros)) {
		macro = TAILQ_FIRST(&parse_macros);
		TAILQ_REMOVE(&parse_macros, macro, entry);

		if (macro->type == MACRO_STRING)
			xfree(macro->value.str);
		xfree(macro);
	}

        fclose(f);
        return (0);
}

__dead void
yyerror(const char *fmt, ...)
{
        va_list  ap;
        char    *s;

	xasprintf(&s,
	    "%s: %s at line %d", parse_file->path, fmt, parse_file->line);

        va_start(ap, fmt);
        log_vwrite(NULL, LOG_CRIT, s, ap);
        va_end(ap);

        exit(1);
}

struct macro *
find_macro(char *name)
{
	struct macro	*macro;

	TAILQ_FOREACH(macro, &parse_macros, entry) {
		if (strcmp(macro->name, name) == 0)
			return (macro);
	}

	return (NULL);
}
#line 123 "parse.y"
#ifndef YYSTYPE_DEFINED
#define YYSTYPE_DEFINED
typedef union
{
        int 	 	 number;
        char 		*string;
	int		 flag;
        struct tags 	*tags;
	struct {
		enum action	 act;
		char 		*str;
	} action;
} YYSTYPE;
#endif /* YYSTYPE_DEFINED */
#line 121 "parse.c"
#define NONE 257
#define TOKMATCH 258
#define TOKIGNORE 259
#define TOKSET 260
#define TOKFILE 261
#define TOKIN 262
#define TOKTAG 263
#define TOKAUTOAPPEND 264
#define TOKOPEN 265
#define TOKAPPEND 266
#define TOKCLOSE 267
#define TOKEXPIRE 268
#define TOKWHEN 269
#define TOKNOT 270
#define TOKCLEAR 271
#define TOKHOURS 272
#define TOKMINUTES 273
#define TOKSECONDS 274
#define OPTMAILCMD 275
#define OPTMAILTIME 276
#define OPTUSER 277
#define OPTGROUP 278
#define OPTCACHEFILE 279
#define OPTPIDFILE 280
#define OPTLOGREGEXP 281
#define OPTMAXTHREADS 282
#define NUMBER 283
#define STRING 284
#define STRMACRO 285
#define NUMMACRO 286
#define STRCOMMAND 287
#define NUMCOMMAND 288
#define TAG 289
#define BASICACTION 290
#define YYERRCODE 256
const short yylhs[] =
	{                                        -1,
    0,    0,    0,    0,    0,    0,    1,    1,    1,    1,
    7,    7,    7,    2,    2,    2,   11,   11,    8,    8,
    9,    9,   14,   14,   14,   14,   14,   14,   14,   14,
    3,    3,    5,    5,    4,    4,    4,    4,    6,    6,
   10,   10,   13,   13,   13,   13,   13,   13,   12,   12,
};
const short yylen[] =
	{                                         2,
    0,    2,    2,    2,    2,    2,    2,    2,    2,    1,
    1,    1,    1,    1,    1,    1,    3,    3,    1,    1,
    1,    1,    3,    3,    3,    3,    3,    3,    3,    3,
    2,    1,    2,    1,    2,    2,    4,    0,    2,    0,
    1,    0,    5,   10,   13,    6,    7,    7,    4,    2,
};
const short yydefred[] =
	{                                      1,
    0,    6,    0,    0,    0,    0,    0,    2,    3,    4,
    5,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   12,   13,   11,    0,    0,    0,   36,   35,    0,
    0,   23,   15,   16,   14,   26,    0,   19,   20,   27,
   21,   22,   28,   24,   25,   29,   30,    0,   17,   18,
   34,    0,    0,    0,    7,    8,    9,   49,   33,   37,
   39,   32,    0,    0,    0,    0,    0,   43,    0,   46,
    0,    0,   31,   41,    0,   47,   48,    0,    0,    0,
    0,    0,   45,
};
const short yydgoto[] =
	{                                       1,
   36,   37,   68,   13,   52,   54,   25,   40,   43,   75,
    8,    9,   10,   11,
};
const short yysindex[] =
	{                                      0,
 -220,    0, -256, -206, -265,  -52,  -38,    0,    0,    0,
    0,  -42, -265, -265, -232, -240, -240, -265, -265, -265,
 -232,    0,    0,    0, -229, -249, -230,    0,    0, -234,
 -203,    0,    0,    0,    0,    0, -215,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, -210,    0,    0,
    0, -123, -265, -241,    0,    0,    0,    0,    0,    0,
    0,    0, -265, -265, -265, -265, -265,    0, -182,    0,
 -254, -254,    0,    0, -185,    0,    0, -232, -254, -184,
 -232, -254,    0,};
const short yyrindex[] =
	{                                      0,
    0,    0, -207,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    7,    0,    0,    0,    0,    0,
 -238,    0,    0,    0,    0,    0,    1,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -181,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   12,
    0,    0,    0,};
const short yygindex[] =
	{                                      0,
    6,  -13,  -40,    0,    0,    0,   -3,    0,    0,    0,
    0,    0,    0,    0,
};
#define YYTABLESIZE 298
const short yytable[] =
	{                                      29,
   10,   60,   38,   41,   62,   12,   50,   47,   26,   31,
   32,   44,   39,   42,   44,   45,   46,   62,   22,   23,
   40,   24,   27,   63,   64,   65,   40,   40,   40,   66,
   76,   77,   40,   48,   49,   67,    2,    3,   80,    4,
    5,   83,   33,   22,   23,   34,   24,   35,   67,   61,
   33,   40,   50,   34,   51,   35,   55,   56,   57,   69,
   70,   71,   72,   73,    6,    7,   53,   82,   14,   15,
   16,   17,   18,   19,   20,   21,   38,   38,   58,   38,
   30,   74,   78,   79,   81,    0,   42,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   59,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   28,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   10,   10,   10,
   10,   10,    0,   50,   50,    0,   50,   50,   44,   44,
    0,   44,   44,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   10,   10,    0,    0,    0,
   10,   50,   50,    0,    0,    0,   44,   44,
};
const short yycheck[] =
	{                                      42,
    0,  125,   16,   17,  259,  262,    0,   21,   61,   13,
   14,    0,   16,   17,   18,   19,   20,  259,  284,  285,
  259,  287,   61,  265,  266,  267,  265,  266,  267,  271,
   71,   72,  271,  263,  284,  290,  257,  258,   79,  260,
  261,   82,  283,  284,  285,  286,  287,  288,  290,   53,
  283,  290,  283,  286,  289,  288,  272,  273,  274,   63,
   64,   65,   66,   67,  285,  286,  270,   81,  275,  276,
  277,  278,  279,  280,  281,  282,  284,  285,  289,  287,
  123,  264,  268,   78,  269,   -1,  268,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  289,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  289,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,  258,  259,
  260,  261,   -1,  257,  258,   -1,  260,  261,  257,  258,
   -1,  260,  261,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  285,  286,   -1,   -1,   -1,
  290,  285,  286,   -1,   -1,   -1,  285,  286,
};
#define YYFINAL 1
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 290
#if YYDEBUG
const char * const yyname[] =
	{
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,"'*'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'='",0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"NONE","TOKMATCH",
"TOKIGNORE","TOKSET","TOKFILE","TOKIN","TOKTAG","TOKAUTOAPPEND","TOKOPEN",
"TOKAPPEND","TOKCLOSE","TOKEXPIRE","TOKWHEN","TOKNOT","TOKCLEAR","TOKHOURS",
"TOKMINUTES","TOKSECONDS","OPTMAILCMD","OPTMAILTIME","OPTUSER","OPTGROUP",
"OPTCACHEFILE","OPTPIDFILE","OPTLOGREGEXP","OPTMAXTHREADS","NUMBER","STRING",
"STRMACRO","NUMMACRO","STRCOMMAND","NUMCOMMAND","TAG","BASICACTION",
};
const char * const yyrule[] =
	{"$accept : cmds",
"cmds :",
"cmds : cmds defmacro",
"cmds : cmds file",
"cmds : cmds rule",
"cmds : cmds set",
"cmds : cmds NONE",
"time : num TOKHOURS",
"time : num TOKMINUTES",
"time : num TOKSECONDS",
"time : num",
"str : STRCOMMAND",
"str : STRING",
"str : STRMACRO",
"num : NUMCOMMAND",
"num : NUMBER",
"num : NUMMACRO",
"defmacro : STRMACRO '=' STRING",
"defmacro : NUMMACRO '=' NUMBER",
"user : num",
"user : str",
"group : num",
"group : str",
"set : TOKSET OPTMAILCMD str",
"set : TOKSET OPTCACHEFILE str",
"set : TOKSET OPTPIDFILE str",
"set : TOKSET OPTMAILTIME time",
"set : TOKSET OPTUSER user",
"set : TOKSET OPTGROUP group",
"set : TOKSET OPTLOGREGEXP str",
"set : TOKSET OPTMAXTHREADS num",
"action : BASICACTION str",
"action : TOKIGNORE",
"tagslist : tagslist TAG",
"tagslist : TAG",
"tags : TOKIN '*'",
"tags : TOKIN TAG",
"tags : TOKIN '{' tagslist '}'",
"tags :",
"not : TOKNOT str",
"not :",
"autoappend : TOKAUTOAPPEND",
"autoappend :",
"rule : TOKMATCH tags str not action",
"rule : TOKMATCH tags str not TOKOPEN str autoappend TOKEXPIRE time action",
"rule : TOKMATCH tags str not TOKOPEN str autoappend TOKEXPIRE time action TOKWHEN num action",
"rule : TOKMATCH tags str not TOKAPPEND str",
"rule : TOKMATCH tags str not TOKCLOSE str action",
"rule : TOKMATCH tags str not TOKCLEAR str action",
"file : TOKFILE str TOKTAG TAG",
"file : TOKFILE str",
};
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
/* LINTUSED */
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
unsigned int yystacksize;
#line 667 "parse.y"

/* Programs */
#line 379 "parse.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(void)
{
    unsigned int newsize;
    long sslen;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    sslen = yyssp - yyss;
#ifdef SIZE_MAX
#define YY_SIZE_MAX SIZE_MAX
#else
#define YY_SIZE_MAX 0xffffffffU
#endif
    if (newsize && YY_SIZE_MAX / newsize < sizeof *newss)
        goto bail;
    newss = yyss ? (short *)realloc(yyss, newsize * sizeof *newss) :
      (short *)malloc(newsize * sizeof *newss); /* overflow check above */
    if (newss == NULL)
        goto bail;
    yyss = newss;
    yyssp = newss + sslen;
    if (newsize && YY_SIZE_MAX / newsize < sizeof *newvs)
        goto bail;
    newvs = yyvs ? (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs) :
      (YYSTYPE *)malloc(newsize * sizeof *newvs); /* overflow check above */
    if (newvs == NULL)
        goto bail;
    yyvs = newvs;
    yyvsp = newvs + sslen;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
bail:
    if (yyss)
            free(yyss);
    if (yyvs)
            free(yyvs);
    yyss = yyssp = NULL;
    yyvs = yyvsp = NULL;
    yystacksize = 0;
    return -1;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
int
yyparse(void)
{
    int yym, yyn, yystate;
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif /* YYDEBUG */

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym)
        yyval = yyvsp[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
    switch (yyn)
    {
case 7:
#line 159 "parse.y"
{
	      if (yyvsp[-1].number > INT_MAX / 3600)
		      yyerror("time too big: %d", yyvsp[-1].number);
	      yyval.number = yyvsp[-1].number * 3600;
      }
break;
case 8:
#line 165 "parse.y"
{
	      if (yyvsp[-1].number > INT_MAX / 60)
		      yyerror("time too big: %d", yyvsp[-1].number);
	      yyval.number = yyvsp[-1].number * 60;
      }
break;
case 10:
#line 172 "parse.y"
{
	      if (yyvsp[0].number > INT_MAX)
		      yyerror("time too big: %d", yyvsp[0].number);
	      yyval.number = yyvsp[0].number;
      }
break;
case 11:
#line 179 "parse.y"
{
	     yyerror("commands are not supported");
	     /*
	     $$ = run_command($1, parse_file->path);
	     xfree($1);
	     */
     }
break;
case 12:
#line 187 "parse.y"
{
	     yyval.string = yyvsp[0].string;
     }
break;
case 13:
#line 191 "parse.y"
{
	     struct macro	*macro;

	     if (strlen(yyvsp[0].string) > MAXNAMESIZE)
		     yyerror("macro name too long: %s", yyvsp[0].string);

	     if ((macro = find_macro(yyvsp[0].string)) == NULL)
		     yyerror("undefined macro: %s", yyvsp[0].string);
	     if (macro->type != MACRO_STRING)
		     yyerror("string macro expected: %s", yyvsp[0].string);

	     yyval.string = xstrdup(macro->value.str);

	     xfree(yyvsp[0].string);
     }
break;
case 14:
#line 208 "parse.y"
{
	     yyerror("commands are not supported");
	     /*
	     const char	*errstr;
	     char	*s;
	     
	     s = run_command($1, parse_file->path);
	     
	     $$ = strtonum(s, 0, LLONG_MAX, &errstr);
	     if (errstr != NULL)
		     yyerror("number is %s", errstr);
	     
	     xfree(s);
	     
	     xfree($1);
	     */
      }
break;
case 15:
#line 226 "parse.y"
{
	     yyval.number = yyvsp[0].number;
     }
break;
case 16:
#line 230 "parse.y"
{
	     struct macro	*macro;

	     if (strlen(yyvsp[0].string) > MAXNAMESIZE)
		     yyerror("macro name too long: %s", yyvsp[0].string);

	     if ((macro = find_macro(yyvsp[0].string)) == NULL)
		     yyerror("undefined macro: %s", yyvsp[0].string);
	     if (macro->type != MACRO_NUMBER)
		     yyerror("number macro expected: %s", yyvsp[0].string);

	     yyval.number = macro->value.num;

	     xfree(yyvsp[0].string);
     }
break;
case 17:
#line 247 "parse.y"
{
		  struct macro	*macro;
		  
		  if ((macro = find_macro(yyvsp[-2].string)) == NULL) {
			  macro = xmalloc(sizeof *macro);
			  if (strlen(yyvsp[-2].string) > MAXNAMESIZE)
				  yyerror("macro name too long: %s", yyvsp[-2].string);
			  strlcpy(macro->name, yyvsp[-2].string, sizeof macro->name);
			  TAILQ_INSERT_HEAD(&parse_macros, macro, entry);
		  }
		  macro->type = MACRO_STRING;
		  macro->value.str = yyvsp[0].string;
		  xfree(yyvsp[-2].string);
	  }
break;
case 18:
#line 262 "parse.y"
{	
	     struct macro	*macro;
	     
	     if ((macro = find_macro(yyvsp[-2].string)) == NULL) {
		     macro = xmalloc(sizeof *macro);
		     if (strlen(yyvsp[-2].string) > MAXNAMESIZE)
			     yyerror("macro name too long: %s", yyvsp[-2].string);
		     strlcpy(macro->name, yyvsp[-2].string, sizeof macro->name);
		     TAILQ_INSERT_HEAD(&parse_macros, macro, entry);
	     }
	     macro->type = MACRO_NUMBER;
	     macro->value.num = yyvsp[0].number;
	     xfree(yyvsp[-2].string);
	  }
break;
case 19:
#line 278 "parse.y"
{
	      struct passwd *pw;
	      
	      pw = getpwuid(yyvsp[0].number);
	      if (pw == NULL)
		      yyerror("unknown uid %d", yyvsp[0].number);
	      
	      yyval.number = pw->pw_uid;
	      
	      endpwent();
      }
break;
case 20:
#line 290 "parse.y"
{
	      struct passwd *pw;
	     
	      pw = getpwnam(yyvsp[0].string);
	      if (pw == NULL)
		      yyerror("unknown user \"%s\"", yyvsp[0].string);
	      
	      yyval.number = pw->pw_uid;
	      
	      endpwent();
	      xfree(yyvsp[0].string);
      }
break;
case 21:
#line 304 "parse.y"
{
	       struct group *gr;

	       gr = getgrgid(yyvsp[0].number);
	       if (gr == NULL)
		       yyerror("unknown gid %d", yyvsp[0].number);

	       yyval.number = gr->gr_gid;
	       
             endgrent();
       }
break;
case 22:
#line 316 "parse.y"
{
	       struct group *gr;

	       gr = getgrnam(yyvsp[0].string);
	       if (gr == NULL)
		       yyerror("unknown group \"%s\"", yyvsp[0].string);
	       
	       yyval.number = gr->gr_gid;

	       endgrent();
	       xfree(yyvsp[0].string);
       }
break;
case 23:
#line 330 "parse.y"
{
             if (conf.mail_cmd != NULL)
                     xfree(conf.mail_cmd);
             conf.mail_cmd = yyvsp[0].string;
     }
break;
case 24:
#line 336 "parse.y"
{
             if (conf.cache_file == NULL)
                     conf.cache_file = yyvsp[0].string;
             else
                     xfree(yyvsp[0].string);
     }
break;
case 25:
#line 343 "parse.y"
{
             if (conf.pid_file == NULL)
                     conf.pid_file = yyvsp[0].string;
             else
                     xfree(yyvsp[0].string);
     }
break;
case 26:
#line 350 "parse.y"
{
             if (yyvsp[0].number < 10)
                     yyerror("mail time must be at least 10 seconds");

             conf.mail_time = yyvsp[0].number;
     }
break;
case 27:
#line 357 "parse.y"
{
             conf.uid = yyvsp[0].number;
     }
break;
case 28:
#line 361 "parse.y"
{
	     conf.gid = yyvsp[0].number;
     }
break;
case 29:
#line 365 "parse.y"
{
	     if (regcomp(&conf.entry_re, yyvsp[0].string, REG_EXTENDED) != 0)
		     yyerror("invalid log regexp: %s", yyvsp[0].string);

             xfree(yyvsp[0].string);
     }
break;
case 30:
#line 372 "parse.y"
{
             if (yyvsp[0].number < 10)
                     yyerror("max threads must be at least 10");
             conf.thr_limit = yyvsp[0].number;
     }
break;
case 31:
#line 379 "parse.y"
{
		yyval.action.act = yyvsp[-1].action.act;
		yyval.action.str = yyvsp[0].string;
        }
break;
case 32:
#line 384 "parse.y"
{
		yyval.action.act = ACT_IGNORE;
		yyval.action.str = NULL;
        }
break;
case 33:
#line 390 "parse.y"
{
		  struct tag	*tag;

		  yyval.tags = yyvsp[-1].tags;

		  tag = xmalloc(sizeof *tag);
		  strlcpy(tag->name, yyvsp[0].string, sizeof tag->name);
		  TAILQ_INSERT_TAIL(yyval.tags, tag, entry);
	  }
break;
case 34:
#line 400 "parse.y"
{
		  struct tag	*tag;
		  
		  yyval.tags = xmalloc(sizeof *yyval.tags);
		  TAILQ_INIT(yyval.tags);
		  
		  tag = xmalloc(sizeof *tag);
		  strlcpy(tag->name, yyvsp[0].string, sizeof tag->name);
		  TAILQ_INSERT_TAIL(yyval.tags, tag, entry);
	  }
break;
case 35:
#line 412 "parse.y"
{
	      yyval.tags = xmalloc(sizeof *yyval.tags);
	      TAILQ_INIT(yyval.tags);
      }
break;
case 36:
#line 417 "parse.y"
{
	      struct tag	*tag;
      
	      yyval.tags = xmalloc(sizeof *yyval.tags);
	      TAILQ_INIT(yyval.tags);

	      tag = xmalloc(sizeof *tag);
	      strlcpy(tag->name, yyvsp[0].string, sizeof tag->name);
	      TAILQ_INSERT_TAIL(yyval.tags, tag, entry);
      }
break;
case 37:
#line 428 "parse.y"
{
	      yyval.tags = yyvsp[-1].tags;
      }
break;
case 38:
#line 432 "parse.y"
{
	      yyval.tags = xmalloc(sizeof *yyval.tags);
	      TAILQ_INIT(yyval.tags);
      }
break;
case 39:
#line 438 "parse.y"
{
	     yyval.string = yyvsp[0].string;
     }
break;
case 40:
#line 442 "parse.y"
{
	     yyval.string = NULL;
     }
break;
case 41:
#line 447 "parse.y"
{
		    yyval.flag = 1;
	    }
break;
case 42:
#line 451 "parse.y"
{
		    yyval.flag = 0;
	    }
break;
case 43:
#line 457 "parse.y"
{
              struct rule *rule;
	      
              rule = add_rule(yyvsp[0].action.act, yyvsp[-3].tags, yyvsp[-2].string, yyvsp[-1].string);
              if (rule == NULL)
                      exit(1);

              rule->params.str = yyvsp[0].action.str;

	      free_tags(yyvsp[-3].tags);
	      xfree(yyvsp[-3].tags);
              xfree(yyvsp[-2].string);
	      if (yyvsp[-1].string != NULL)
		      xfree(yyvsp[-1].string);
      }
break;
case 44:
#line 475 "parse.y"
{
              struct rule *rule;

              if (*yyvsp[-4].string == '\0')
                      yyerror("context key cannot be empty string");
	      if (yyvsp[-1].number == 0)
                      yyerror("expiry time cannot be zero");

              rule = add_rule(ACT_OPEN, yyvsp[-8].tags, yyvsp[-7].string, yyvsp[-6].string);
              if (rule == NULL)
                      exit(1);

              rule->params.key = yyvsp[-4].string;

	      rule->params.exp_act = yyvsp[0].action.act;
	      rule->params.exp_str = yyvsp[0].action.str;
              rule->params.exp_time = yyvsp[-1].number;

	      if (yyvsp[-3].flag) {
		      rule = add_rule(ACT_APPEND, yyvsp[-8].tags, yyvsp[-7].string, yyvsp[-6].string);
		      if (rule == NULL)
			      exit(1);
		      rule->params.key = xstrdup(yyvsp[-4].string);
	      }

	      free_tags(yyvsp[-8].tags);
	      xfree(yyvsp[-8].tags);
              xfree(yyvsp[-7].string);
	      if (yyvsp[-6].string != NULL)
		      xfree(yyvsp[-6].string);
      }
break;
case 45:
#line 510 "parse.y"
{
              struct rule 	*rule;

              if (*yyvsp[-7].string == '\0')
                      yyerror("context key cannot be empty string");
              if (yyvsp[-4].number == 0)
                      yyerror("expiry time cannot be zero");
              if (yyvsp[-1].number == 0)
                      yyerror("number of entries cannot be zero");

              rule = add_rule(ACT_OPEN, yyvsp[-11].tags, yyvsp[-10].string, yyvsp[-9].string);
              if (rule == NULL)
                      exit(1);

              rule->params.key = yyvsp[-7].string;

	      rule->params.exp_act = yyvsp[-3].action.act;
	      rule->params.exp_str = yyvsp[-3].action.str;
              rule->params.exp_time = yyvsp[-4].number;

              rule->params.ent_max = yyvsp[-1].number;
	      rule->params.ent_act = yyvsp[0].action.act;
	      rule->params.ent_str = yyvsp[0].action.str;

	      if (yyvsp[-6].flag) {
		      rule = add_rule(ACT_APPEND, yyvsp[-11].tags, yyvsp[-10].string, yyvsp[-9].string);
		      if (rule == NULL)
			      exit(1);
		      rule->params.key = xstrdup(yyvsp[-7].string);
	      }

	      free_tags(yyvsp[-11].tags);
	      xfree(yyvsp[-11].tags);
              xfree(yyvsp[-10].string);
	      if (yyvsp[-9].string != NULL)
		      xfree(yyvsp[-9].string);
      }
break;
case 46:
#line 550 "parse.y"
{
              struct rule 	*rule;

              if (*yyvsp[0].string == '\0')
                      yyerror("context key cannot be empty string");

              rule = add_rule(ACT_APPEND, yyvsp[-4].tags, yyvsp[-3].string, yyvsp[-2].string);
              if (rule == NULL)
                      exit(1);

              rule->params.key = yyvsp[0].string;

	      free_tags(yyvsp[-4].tags);
	      xfree(yyvsp[-4].tags);
              xfree(yyvsp[-3].string);
	      if (yyvsp[-2].string != NULL)
		      xfree(yyvsp[-2].string);
      }
break;
case 47:
#line 571 "parse.y"
{
              struct rule 	*rule;

              if (*yyvsp[-1].string == '\0')
                      yyerror("context key cannot be empty string");

              rule = add_rule(ACT_CLOSE, yyvsp[-5].tags, yyvsp[-4].string, yyvsp[-3].string);
              if (rule == NULL)
                      exit(1);

              rule->params.key = yyvsp[-1].string;

	      rule->params.close_act = yyvsp[0].action.act;
	      rule->params.close_str = yyvsp[0].action.str;

	      free_tags(yyvsp[-5].tags);
	      xfree(yyvsp[-5].tags);
              xfree(yyvsp[-4].string);
	      if (yyvsp[-3].string != NULL)
		      xfree(yyvsp[-3].string);
      }
break;
case 48:
#line 595 "parse.y"
{
              struct rule 	*rule;

              if (*yyvsp[-1].string == '\0')
                      yyerror("context key cannot be empty string");

              rule = add_rule(ACT_CLEAR, yyvsp[-5].tags, yyvsp[-4].string, yyvsp[-3].string);
              if (rule == NULL)
                      exit(1);

              rule->params.key = yyvsp[-1].string;

	      rule->params.clear_act = yyvsp[0].action.act;
	      rule->params.clear_str = yyvsp[0].action.str;

	      free_tags(yyvsp[-5].tags);
	      xfree(yyvsp[-5].tags);
              xfree(yyvsp[-4].string);
	      if (yyvsp[-3].string != NULL)
		      xfree(yyvsp[-3].string);
      }
break;
case 49:
#line 618 "parse.y"
{
              if (*yyvsp[-2].string == '\0')
                      yyerror("path cannot be empty string");

              if (add_file(yyvsp[-2].string, yyvsp[0].string) == NULL)
                      exit(1);

              xfree(yyvsp[-2].string);
      }
break;
case 50:
#line 628 "parse.y"
{
              unsigned int	n;
              char		name[MAXTAGLEN], num[13], *ptr;
	      size_t		len, nlen;

              if (*yyvsp[0].string == '\0')
                      yyerror("path cannot be empty string");

	      if ((ptr = basename(yyvsp[0].string)) != NULL)
		      strlcpy(name, ptr, sizeof name);
	      else 
		      strlcpy(name, "__", sizeof name);
	      len = strlen(name);

	      if (find_file_by_tag(name)) {
		      for (n = 1; n > 0; n++) {
			      snprintf(num, sizeof num, "%u", n);
			      nlen = strlen(num);
			      
			      if (len + nlen > sizeof name)
				      name[(sizeof name) - nlen - 1] = '\0';
			      else
				      name[len - 1] = '\0';

			      strlcat(name, num, sizeof name);
			      log_debug("testing tag: %s (%u)", name, n);
			      
			      if (!find_file_by_tag(name))
				      break;
		      }
		      if (n == 0)
			      yyerror("unable to find unused tag");
	      }
	      if (add_file(yyvsp[0].string, name) == NULL)
		      exit(1);
              xfree(yyvsp[0].string);
      }
break;
#line 1140 "parse.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    if (yyss)
            free(yyss);
    if (yyvs)
            free(yyvs);
    yyss = yyssp = NULL;
    yyvs = yyvsp = NULL;
    yystacksize = 0;
    return (1);
yyaccept:
    if (yyss)
            free(yyss);
    if (yyvs)
            free(yyvs);
    yyss = yyssp = NULL;
    yyvs = yyvsp = NULL;
    yystacksize = 0;
    return (0);
}
